#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "polygon.h"

/*
 * generic interface for polygon. all functions simply call the proper implementation specified in given polygon structure
 */
polygon_t polygon_new(polygon_ops_t ops,char *spec)
{
    polygon_t ret = NULL;

    if ( ops == NULL || spec == NULL ) return NULL;

    ret = (polygon_t)malloc(sizeof(struct polygon));
    assert(ret != NULL);
    ret->ops = ops;
    ret->ctx = ops->op_create();
    assert(ret->ctx != NULL);
    ops->op_init(ret->ctx,spec);

    return ret;
}

void polygon_free(polygon_t p)
{
    if ( p == NULL ) return;
    if ( p->ctx != NULL && p->ops != NULL ) {
        p->ops->op_destroy(p->ctx);
    }
    p->ctx = NULL;
    p->ops = NULL;
    free(p);
}

inline double polygon_get_area(polygon_t p)
{
    if ( p == NULL || p->ctx == NULL || p->ops == NULL )
        return .0;

    return p->ops->op_area(p->ctx);
}

inline void polygon_fill(polygon_t p)
{
    if ( p == NULL || p->ctx == NULL || p->ops == NULL )
        return;

    p->ops->op_fill(p->ctx);
}

